"""
    Testing dataset
"""

import pandas as pd
import pytest
from relevanceai.clusterer import kmeans_clusterer
from relevanceai.http_client import Dataset, Client, ClusterOps
from relevanceai.dataset_api.cluster_groupby import ClusterGroupby

CLUSTER_ALIAS = "kmeans_10"
VECTOR_FIELDS = ["sample_1_vector_"]


@pytest.fixture
def test_clusterer(test_client: Client, clustered_dataset_id: Dataset):
    df: Dataset = test_client.Dataset(clustered_dataset_id)

    model = get_model()

    clusterer: ClusterOps = df.cluster(
        model=model, vector_fields=VECTOR_FIELDS, alias=CLUSTER_ALIAS
    )
    return clusterer


def get_model():
    # get a kmeans model
    from relevanceai.clusterer.kmeans_clusterer import KMeansModel

    return KMeansModel(verbose=False)


def test_cluster(test_df: Dataset):

    vector_field = "sample_1_vector_"
    alias = "test_alias"

    model = get_model()

    test_df.cluster(
        model=model, alias=alias, vector_fields=[vector_field], overwrite=True
    )
    assert f"_cluster_.{vector_field}.{alias}" in test_df.schema


def test_closest(test_clusterer: ClusterOps):
    closest = test_clusterer.list_closest_to_center()
    assert len(closest["results"]) > 0


def test_furthest(test_clusterer: ClusterOps):
    furthest = test_clusterer.list_furthest_from_center()
    assert len(furthest["results"]) > 0


def test_agg(test_clusterer: ClusterOps):
    agg = test_clusterer.agg({"sample_2_value": "avg"})
    cluster_groupby: ClusterGroupby = test_clusterer.groupby(["sample_3_description"])
    groupby_agg = cluster_groupby.agg({"sample_2_value": "avg"})
    assert isinstance(groupby_agg, dict)
    assert len(groupby_agg) > 0


def test_agg_std(test_clusterer: ClusterOps):
    agg = test_clusterer.agg({"sample_2_value": "avg"})
    cluster_groupby: ClusterGroupby = test_clusterer.groupby(["sample_3_description"])
    groupby_agg = cluster_groupby.agg({"sample_2_value": "std_deviation"})
    assert isinstance(groupby_agg, dict)
    assert len(groupby_agg) > 0



# -*- coding: utf-8 -*-
import warnings
from relevanceai import vector_tools
from relevanceai.http_client import Client

# Cluster _Base Utilities
from relevanceai.clusterer.cluster_base import ClusterBase, CentroidClusterBase
from relevanceai.clusterer.clusterer import ClusterOps
import requests

# Import useful utility if possible as well
try:
    from jsonshower import show_json
except ModuleNotFoundError:
    pass

__version__ = "1.1.5"

try:
    pypi_data = requests.get("https://pypi.org/pypi/relevanceai/json").json()
    pypi_info = pypi_data.get("info", None)
    if pypi_info is not None:
        latest_version = pypi_info.get("version", None)
    else:
        latest_version = None

    if __version__ != latest_version and latest_version is not None:
        changelog_url: str = (
            f"https://relevanceai.readthedocs.io/en/{__version__}/changelog.html"
        )
        MESSAGE = """We noticed you don't have the latest version! 
We recommend updating to the latest version ({latest_version}) to get all bug fixes and newest features!
You can do this by running pip install -U relevanceai. 
Changelog: {changelog_url}.""".format(  # type: ignore
            version=__version__,
            latest_version=latest_version,
            changelog_url=changelog_url,
        )
        warnings.warn(MESSAGE)
except:
    pass



"""All fixtures go here.
"""
import os
import pytest

import pandas as pd

from relevanceai import Client
from relevanceai.dataset_api import Dataset

import tempfile

from tests.globals.constants import *
from tests.globals.document import *
from tests.globals.documents import *
from tests.globals.objects import *
from tests.globals.datasets import *
from tests.globals.clusterers import *

REGION = os.getenv("TEST_REGION")


# def pytest_sessionstart(session):
#     """
#     Pytest's configuration
#     """
#     # Deleting all mixpanel analytics from tests
#     CONFIG_FN = os.path.join("relevanceai", "config.ini")
#     with open(CONFIG_FN, "r") as f:
#         lines = f.readlines()

#     os.remove(CONFIG_FN)

#     with open(CONFIG_FN, "w") as f:
#         for i, line in enumerate(lines):
#             if i < 27:
#                 f.write(line)


@pytest.fixture(scope="session")
def test_project():
    if REGION == "us-east-1":
        return os.getenv("TEST_US_PROJECT")
    return os.getenv("TEST_PROJECT")


@pytest.fixture(scope="session")
def test_api_key():
    if REGION == "us-east-1":
        return os.getenv("TEST_US_API_KEY")
    return os.getenv("TEST_API_KEY")


@pytest.fixture(scope="session")
def test_firebase_uid():
    return "relevanceai-sdk-test-user"


def correct_client_config(client):
    client.config.reset()
    if client.region != "us-east-1":
        raise ValueError("default value aint RIGHT")


@pytest.fixture(scope="session")
def test_client(test_project, test_api_key, test_firebase_uid):
    if REGION is None:
        client = Client(
            project=test_project, api_key=test_api_key, firebase_uid=test_firebase_uid
        )
    else:
        client = Client(
            project=test_project,
            api_key=test_api_key,
            firebase_uid=test_firebase_uid,
            region=REGION,
        )
    # For some reason not resetting to default
    correct_client_config(client)
    return client


@pytest.fixture(scope="module")
def test_csv_dataset(test_client: Client, vector_documents: List[Dict]):
    correct_client_config(test_client)
    test_dataset_id = generate_dataset_id()

    with tempfile.NamedTemporaryFile(mode="w", delete=False) as csvfile:
        df = pd.DataFrame(vector_documents)
        df.to_csv(csvfile)

        response = test_client._insert_csv(test_dataset_id, csvfile.name)
        yield response, len(vector_documents)
        test_client.datasets.delete(test_dataset_id)


@pytest.fixture(scope="module")
def test_read_df(test_client: Client, vector_documents: List[Dict]):
    correct_client_config(test_client)
    DATASET_ID = "_sample_df_"
    df = test_client.Dataset(DATASET_ID)
    results = df.upsert_documents(vector_documents)
    yield results


@pytest.fixture(scope="module")
def test_csv_df(test_df: Dataset, vector_documents: List[Dict]):
    """Sample csv dataset"""
    test_df.config.reset()
    with tempfile.NamedTemporaryFile(mode="w", delete=False) as csvfile:
        df = pd.DataFrame(vector_documents)
        df.to_csv(csvfile)

        response = test_df.insert_csv(csvfile.name)
        yield response, len(vector_documents)



"""
Sklearn Integration Test
"""
from relevanceai import Client
from relevanceai.dataset_api import Dataset

from tests.globals.constants import generate_random_string


def test_cluster(test_df: Dataset):
    from sklearn.cluster import KMeans

    vector_field = "sample_1_vector_"
    alias = generate_random_string().lower()

    # check they're not in first
    assert f"_cluster_.{vector_field}.{alias}" not in test_df.schema

    model = KMeans()
    clusterer = test_df.cluster(
        model=model, alias=alias, vector_fields=[vector_field], overwrite=True
    )
    assert f"_cluster_.{vector_field}.{alias}" in test_df.schema
    assert len(clusterer.list_closest_to_center()) > 0


def test_dbscan(test_client: Client, test_df: Dataset):
    from sklearn.cluster import DBSCAN

    ALIAS = "dbscan"

    model = DBSCAN()
    clusterer = test_client.ClusterOps(alias=ALIAS, model=model)
    clusterer.fit_predict_update(test_df, ["sample_3_vector_"])
    assert any([x for x in test_df.schema if ALIAS in x])



import pytest

from relevanceai.http_client import Dataset

from .utils import VECTOR_FIELDS


@pytest.fixture(scope="function")
def minibatch_clusterer(test_df: Dataset):
    clusterer = test_df.auto_cluster("minibatchkmeans-20", vector_fields=VECTOR_FIELDS)
    yield clusterer
    clusterer.delete_centroids(test_df.dataset_id, VECTOR_FIELDS)



"""Test DR on cluster
"""
from relevanceai import Client

# def test_dr_on_cluster():
#     # Run dim reduction d
#     from relevanceai.datasets import mock_documents

#     docs = mock_documents()
#     # Reduce dimensions

#     ds = test_client.Dataset("sample456")
#     ds.insert_documents(docs)
#     ds.auto_reduce_dimensions("pca-3", vector_fields=["sample_1_vector_"])
#     ds.auto_cluster("kmeans-2", vector_fields=["_dr_.pca-3.sample_1_vector_"])
#     ds.delete()
#     assert True



import warnings


def warn_function_is_work_in_progress():
    """

    This function is a work in progress.

    """
    warnings.warn(
        "This function is currently a work in progress and is liable to change."
    )



from relevanceai.dataset_api import Dataset

from tests.globals.constants import generate_random_vector

MULTIVECTOR_QUERY = [
    {"vector": generate_random_vector(), "fields": ["sample_1_vector_"]}
]

CHUNK_MULTIVECTOR_QUERY = [
    {"vector": generate_random_vector(), "fields": ["_chunk_.label_chunkvector_"]}
]


def test_vector_search(test_df: Dataset):
    test_df.vector_search(multivector_query=MULTIVECTOR_QUERY)
    assert True


def test_hybrid_search(test_df: Dataset):
    test_df.hybrid_search(
        multivector_query=MULTIVECTOR_QUERY, text="hey", fields=["sample_1_label"]
    )
    assert True


def test_chunk_search(test_df: Dataset):
    test_df.chunk_search(
        multivector_query=CHUNK_MULTIVECTOR_QUERY, chunk_field="_chunk_"
    )
    assert True


def test_multistep_chunk_search(test_df: Dataset):
    test_df.multistep_chunk_search(
        multivector_query=CHUNK_MULTIVECTOR_QUERY,
        first_step_multivector_query=MULTIVECTOR_QUERY,
        chunk_field="_chunk_",
    )
    assert True



import analytics
import asyncio
import json
from typing import Callable

from functools import wraps

from relevanceai.config import CONFIG
from relevanceai.json_encoder import json_encoder
from relevanceai.logger import FileLogger


def enable_tracking():
    if CONFIG.is_field("mixpanel.enable_tracking", CONFIG.config):
        return CONFIG.get_field("mixpanel.enable_tracking", CONFIG.config)


def get_json_size(json_obj):
    # Returns it in bytes
    return len(json.dumps(json_obj).encode("utf-8")) / 1024


def track(func: Callable):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            if enable_tracking():

                async def send_analytics():
                    user_id = args[0].firebase_uid
                    event_name = f"pysdk-{func.__name__}"
                    kwargs.update(dict(zip(func.__code__.co_varnames, args)))
                    properties = {
                        "args": args,
                        "kwargs": kwargs,
                    }
                    if user_id is not None:
                        # Upsert/inserts/updates are too big to track
                        if (
                            "insert" in event_name
                            or "upsert" in event_name
                            or "update" in event_name
                            or "fit" in event_name
                            or "predict" in event_name
                            or get_json_size(
                                json_encoder(properties, force_string=True)
                            )
                            > 30
                        ):
                            analytics.track(
                                user_id=user_id,
                                event=event_name,
                            )
                        else:
                            analytics.track(
                                user_id=user_id,
                                event=event_name,
                                properties=json_encoder(properties, force_string=True),
                            )

                asyncio.ensure_future(send_analytics())
        except Exception as e:
            pass

        return func(*args, **kwargs)

    return wrapper


def identify(func: Callable):
    def wrapper(*args, **kwargs):
        try:
            if enable_tracking():
                user_id = args[0].firebase_uid
                region = args[0].region
                traits = {
                    "region": region,
                }
                if user_id is not None:
                    analytics.identify(user_id, traits)
        except Exception as e:
            pass

        return func(*args, **kwargs)

    return wrapper



from relevanceai.http_client import ClusterOps


def test_minibatch_clusterer(minibatch_clusterer: ClusterOps):
    assert len(minibatch_clusterer.list_closest_to_center()) > 0
    assert len(minibatch_clusterer.centroids) > 0


def test_minibatchkmeans_cluster_report(minibatch_clusterer: ClusterOps):
    assert len(minibatch_clusterer.report()) > 0


def test_kmeans_clusterer(kmeans_clusterer: ClusterOps):
    assert len(kmeans_clusterer.list_closest_to_center()) > 0
    assert len(kmeans_clusterer.centroids) > 0


def test_kmeans_cluster_report(kmeans_clusterer: ClusterOps):
    assert len(kmeans_clusterer.report()) > 0



from relevanceai.http_client import Client, Dataset, ClusterOps


def test_dataset_api_kmeans_centroids_properties(test_client: Client, test_df: Dataset):

    alias: str = "test_alias"
    vector_field: str = "sample_1_vector_"

    from relevanceai.clusterer import KMeansModel

    model = KMeansModel()

    clusterer: ClusterOps = test_client.ClusterOps(model=model, alias=alias)
    clusterer.fit_predict_update(dataset=test_df, vector_fields=[vector_field])

    assert f"_cluster_.{vector_field}.{alias}" in test_df.schema

    # TODO: see why centroids fail
    centroids = clusterer.list_closest_to_center()
    assert len(centroids) > 0
